"""
symbolite.impl.liblatex
~~~~~~~~~~~~~~~~~~~~~

Translate symbolite
into values and functions defined in Python into latex.

:copyright: 2023 by Symbolite Authors, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
"""

from .. import Kind
from . import lang, real

KIND = Kind.CODE
__all__ = ["real", "lang"]
